==> veracity-source-1.0.0.10517.tar.gz: veracity/src/server_files/core/textCache.js <==
/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

function textCache(request, dags, contentType, files)
{
	this.etag = "";
	this.cachefn = "";
	this.oldetag = "";
	this.contentType = contentType || CONTENT_TYPE__JSON;

	this.oldetag = request.headers["If-None-Match"] || "";

	var urlHashStrs = 
		[
			request.requestMethod,
			request.uri,
			this.normalizeQs(request.queryString)
		];
	var dagHashStrs = [];

	if (files)
	{
		// nothing yet
		for ( var i = 0; i < files.length; ++i )
		{
			var fn = files[i];

			var st = sg.fs.stat(fn);
			var mt = st.modtime.getTime().toString();

			dagHashStrs.push(mt);
		}
	}
	else
	{
		dags = dags || [ sg.dagnum.VERSION_CONTROL ];

		for ( var i = 0; i < dags.length; ++i )
		{
			var passdag = "" + dags[i];
			
			var leaves = request.repo.fetch_dag_leaves(passdag);

			for ( var j in leaves )
				dagHashStrs.push(leaves[j]);
		}
	}

	var urlHash = sg.hash(urlHashStrs);

	this.etag = urlHash + "." + sg.hash(dagHashStrs);

	var cachedir = sg.local_settings().machine.server.cache;

	if (cachedir)
	{
		if (! cachedir.match(/[\\\/]$/))
			cachedir += "/";
	}
	else
	{
		cachedir = sg.server_state_dir();

		if (! cachedir.match(/[\\\/]$/))
			cachedir += "/";

		cachedir += "vvcache/";
	}

	if (! sg.fs.exists(cachedir))
	{
		sg.fs.mkdir(cachedir);
		sg.fs.chmod(cachedir, 0777);
	}

	this.cleanOldCache(cachedir, urlHash, this.etag);

	this.cachefn = cachedir + this.etag;
};

textCache.prototype.cleanOldCache = function(cachedir, prefix, currentfn)
{
	if (! sg.fs.exists(cachedir))
		return;

	try
	{
		prefix += ".";
		
		var files = sg.fs.readdir(cachedir);

		for ( var i = 0; i < files.length; ++i )
		{
			var fn = files[i].name;
			
			if ((fn.substring(0, prefix.length) == prefix) &&
				(fn != currentfn)
			   )
			{
				sg.fs.remove(cachedir + "/" + fn);
			}
		}
	}
	catch(e)
	{
		// ignore occasional errors trying to stat a file-in-progress
	}
};

textCache.prototype.isCached = function() 
{
	return( this.etag && sg.fs.exists( this.cachefn ) );
};

textCache.prototype.cacheMatch = function()
{
	return( (this.etag == this.oldetag) && this.isCached() );
};

textCache.prototype.cachedText= function()
{
	if (! this.isCached())
		return("");
	
	return( sg.file.read( this.cachefn ));
};

textCache.prototype.cache = function(text) 
{
	sg.file.write(this.cachefn, text);
	sg.fs.chmod(this.cachefn, 0666);
	this.oldetag = this.etag;
};

textCache.prototype.response = function()
{
	var response = {};

	response.sz = sg.to_sz(this.cachedText());
	response.statusCode = STATUS_CODE__OK;
	response.headers = {
		"Content-Length": response.sz.length,
		"Content-Type": this.contentType,
		"ETag": this.etag
	};
	response.onChunk = function () {
		var ret = response.sz;
		response.sz = null;
		return ret;
	};
	response.finalize = function () {
		response.sz.fin();
		response.sz = null;
	};

	return response;
};

textCache.prototype.normalizeQs = function(qs)
{
	var result = '';
	var parts = vv.parseQueryString(qs);

	delete parts['_'];

	var keys = [];

	for ( var vari in parts )
		keys.push(vari);

	keys.sort();

	for ( var i = 0; i < keys.length; ++i )
	{
		if (result)
			result += "&";
		result += keys[i] + "=" + parts[keys[i]];
	}

	return(result);
};


registerRoutes({
	"/vvscripts.js":
	{
		"GET": 
		{
			onDispatch: function (request)
			{
				var q = request.queryString || "";		
				var params = vv.parseQueryString(q);

				var fns = params.s.split(",");

				var base = sg.local_settings().machine.server.files;

				if (! base.match(/[\\\/]$/))
					base += "/";

				base += "ui/";

				var files = [];

				for ( var i = 0; i < fns.length; ++i )
					files.push(base + fns[i]);

				var cc = new textCache(request, null, CONTENT_TYPE__JAVASCRIPT, files);

				if (! cc.isCached())
				{
					var result = "";

					var realFiles = [];

					for ( i = 0; i < files.length; ++i )
					{
						var fn = files[i];

						var txt = sg.file.read(fn);

						result += txt + "\r\n";
					}

					cc.cache(result);
				}
				else if (cc.cacheMatch())
					return(
						{
							statusCode: STATUS_CODE__NOT_MODIFIED,
							headers: {
								"Content-Length": 0
							}
						}
					);

				return( cc.response() );
			}
		}
	}
});

==> veracity-source-1.1.0.10565.tar.gz: veracity/src/server_files/core/textCache.js <==
/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

function textCache(request, dags, contentType, files)
{
	this.etag = "";
	this.cachefn = "";
	this.oldetag = "";
	this.contentType = contentType || CONTENT_TYPE__JSON;

	this.oldetag = request.headers["If-None-Match"] || "";

	var urlHashStrs = 
		[
			request.requestMethod,
			request.uri,
			this.normalizeQs(request.queryString)
		];
	var dagHashStrs = [];

	if (files)
	{
		// nothing yet
		for ( var i = 0; i < files.length; ++i )
		{
			var fn = files[i];

			var st = sg.fs.stat(fn);
			var mt = st.modtime.getTime().toString();

			dagHashStrs.push(mt);
		}
	}
	else
	{
		dags = dags || [ sg.dagnum.VERSION_CONTROL ];

		for ( var i = 0; i < dags.length; ++i )
		{
			var passdag = "" + dags[i];
			
			var leaves = request.repo.fetch_dag_leaves(passdag);

			for ( var j in leaves )
				dagHashStrs.push(leaves[j]);
		}
	}

	var urlHash = sg.hash(urlHashStrs);

	this.etag = urlHash + "." + sg.hash(dagHashStrs);

	var cachedir = sg.local_settings().machine.server.cache;

	if (cachedir)
	{
		if (! cachedir.match(/[\\\/]$/))
			cachedir += "/";
	}
	else
	{
		cachedir = sg.server_state_dir();

		if (! cachedir.match(/[\\\/]$/))
			cachedir += "/";

		cachedir += "vvcache/";
	}

	if (! sg.fs.exists(cachedir))
	{
		sg.fs.mkdir(cachedir);
		sg.fs.chmod(cachedir, 0777);
	}

	this.cleanOldCache(cachedir, urlHash, this.etag);

	this.cachefn = cachedir + this.etag;
};

textCache.prototype.cleanOldCache = function(cachedir, prefix, currentfn)
{
	if (! sg.fs.exists(cachedir))
		return;

	try
	{
		prefix += ".";
		
		var files = sg.fs.readdir(cachedir);

		for ( var i = 0; i < files.length; ++i )
		{
			var fn = files[i].name;
			
			if ((fn.substring(0, prefix.length) == prefix) &&
				(fn != currentfn)
			   )
			{
				sg.fs.remove(cachedir + "/" + fn);
			}
		}
	}
	catch(e)
	{
		// ignore occasional errors trying to stat a file-in-progress
	}
};

textCache.prototype._isCached = function() 
{
	return( this.etag && sg.fs.exists( this.cachefn ) );
};

textCache.prototype.isCached = function()
{
	return( this._isCached() );
};

textCache.prototype.cacheMatch = function()
{
	return( (this.etag == this.oldetag) && this._isCached() );
};

textCache.prototype.cachedText= function()
{
	if (! this._isCached())
		return("");
	
	return( sg.file.read( this.cachefn ));
};

textCache.prototype.cache = function(text) 
{
	sg.file.write(this.cachefn, text);
	sg.fs.chmod(this.cachefn, 0666);
	this.oldetag = this.etag;
};

textCache.prototype.response = function()
{
	var response = {};

	response.sz = sg.to_sz(this.cachedText());
	response.statusCode = STATUS_CODE__OK;
	response.headers = {
		"Content-Length": response.sz.length,
		"Content-Type": this.contentType,
		"ETag": this.etag
	};
	response.onChunk = function () {
		var ret = response.sz;
		response.sz = null;
		return ret;
	};
	response.finalize = function () {
		response.sz.fin();
		response.sz = null;
	};

	return response;
};

textCache.prototype.normalizeQs = function(qs)
{
	var result = '';
	var parts = vv.parseQueryString(qs);

	delete parts['_'];

	var keys = [];

	for ( var vari in parts )
		keys.push(vari);

	keys.sort();

	for ( var i = 0; i < keys.length; ++i )
	{
		if (result)
			result += "&";
		result += keys[i] + "=" + parts[keys[i]];
	}

	return(result);
};


registerRoutes({
	"/vvscripts.js":
	{
		"GET": 
		{
			onDispatch: function (request)
			{
				var q = request.queryString || "";		
				var params = vv.parseQueryString(q);

				var fns = params.s.split(",");

				var base = sg.local_settings().machine.server.files;

				if (! base.match(/[\\\/]$/))
					base += "/";

				var coreBase = base + "ui/";

				var files = [];

				for ( var i = 0; i < fns.length; ++i )
				{
					var fn = fns[i];
					files.push(coreBase + fn);
				}

				var cc = new textCache(request, null, CONTENT_TYPE__JAVASCRIPT, files);

				if (! cc._isCached())
				{
					var result = "";

					var realFiles = [];

					for ( i = 0; i < files.length; ++i )
					{
						var fn = files[i];

						var txt = sg.file.read(fn);

						if (enableDiagnostics)
							result += txt + "\n";
						else
						{
							var mini = sg.jsmin(txt);

							result += mini + "\n";
						}
					}

					cc.cache(result);
				}
				else if (cc.cacheMatch())
					return(
						{
							statusCode: STATUS_CODE__NOT_MODIFIED,
							headers: {
								"Content-Length": 0
							}
						}
					);

				return( cc.response() );
			}
		}
	}
});
